// CA simulator - hexagonal
// Jacob Peck
// 20101018

import java.util.Random;
import java.util.Scanner;

public class WorldHexChord {
  private CellHexChord[][] currentworld; // current state
  private CellHexChord[][] oldworld; // previous state
  private int x, y;
  private String scale;
  
  // constructor
  public WorldHexChord(int width, int height, String scale) {
    // initialize member variables
    y = height;
    x = width;
    this.scale = scale;
    currentworld = new CellHexChord[width][height];
    oldworld = new CellHexChord[width][height];
    
    // populate states
    for (int i = 0; i < x; i++) {
      for (int j = 0; j < y; j++) {
        currentworld[i][j] = new CellHexChord(0);
        oldworld[i][j] = new CellHexChord(0);
      }
    }
    // assign neighbors
    assignNeighbors();
  }
  
  // randomize the world
  public void randomizeWorld(int numberofstates) {
    Random random = new Random();
    for (int i = 0; i < x; i++) {
      for (int j = 0; j < y; j++) {
        currentworld[i][j].setState(random.nextInt(numberofstates));
      }
    }
  }
  
  // helper function for a 2-state world, such as Conway's GoL
  public void randomizeWorld() {
    randomizeWorld(2);
  }
  
  // assign neighbors to the cells
  // internal representation             external object
  //      [1][2]                                [1] [2]
  //   [3][4][5]                              [3] [4] [5]
  //      [6][7]                                [6] [7]
  // 
  private void assignNeighbors() {
    //System.out.println("x = " + x + " y = " + y);
    for (int i = 0; i < x; i++) {
      for (int j = 0; j < y; j++) {
        System.out.print(".");
        // just rote assignments
        // i-1, j
        if (i > 0) {
          oldworld[i][j].addNeighbor(oldworld[i-1][j]);
          currentworld[i][j].addNeighbor(currentworld[i-1][j]);  
        } else { // stitch right to left
          oldworld[i][j].addNeighbor(oldworld[x-1][j]);
          currentworld[i][j].addNeighbor(currentworld[x-1][j]);
        }
        // i, j-1
        if (j > 0) {
          oldworld[i][j].addNeighbor(oldworld[i][j-1]);
          currentworld[i][j].addNeighbor(currentworld[i][j-1]);
        } else { // stitch top to bottom
          oldworld[i][j].addNeighbor(oldworld[i][y-1]);
          currentworld[i][j].addNeighbor(currentworld[i][y-1]);
        }
        // i, j+1
        if (j < y-1) {
          oldworld[i][j].addNeighbor(oldworld[i][j+1]);
          currentworld[i][j].addNeighbor(currentworld[i][j+1]);
        } else { // stitch bottom to top
          oldworld[i][j].addNeighbor(oldworld[i][0]);
          currentworld[i][j].addNeighbor(currentworld[i][0]);
        }
        // i+1, j-1
        if (i < x-1 && j > 0) {
          oldworld[i][j].addNeighbor(oldworld[i+1][j-1]);
          currentworld[i][j].addNeighbor(currentworld[i+1][j-1]); 
        } else { // stitch top to bottom and left to right
          oldworld[i][j].addNeighbor(oldworld[0][y-1]);
          currentworld[i][j].addNeighbor(currentworld[0][y-1]);
        }
        // i+1, j
        if (i < x-1) {
          oldworld[i][j].addNeighbor(oldworld[i+1][j]);
          currentworld[i][j].addNeighbor(currentworld[i+1][j]);
        } else { // stitch left to right
          oldworld[i][j].addNeighbor(oldworld[0][j]);
          currentworld[i][j].addNeighbor(currentworld[0][j]);
        }
        // i+1, j+1
        if (i < x-1 && j < y-1) {
          oldworld[i][j].addNeighbor(oldworld[i+1][j+1]);
          currentworld[i][j].addNeighbor(currentworld[i+1][j+1]);  
        } else { // stitch bottom to top and left to right
          oldworld[i][j].addNeighbor(oldworld[0][0]);
          currentworld[i][j].addNeighbor(currentworld[0][0]);
        }
      }
    }
  }
  
  public void iterate() {
    for (int i = 0; i < x; i++) {
      for (int j = 0; j < y; j++) {
        // set history
        oldworld[i][j].setState(currentworld[i][j].getState());
      }
    }
    
    for (int i = 0; i < x; i++) {
      for (int j = 0; j < y; j++) {
        // determine whether cell lives or dies
        int state = oldworld[i][j].getState();
        int count = oldworld[i][j].livingNeighbors();
        int newstate = 0;
        // Peck's rules (hex):
        // 2 states, dead(0) and alive(0)
        // Hexagonal lattice, neighborhood is the six closest adjacent cells
        // an alive cell remains alive if it has 2 or 3 live neighbors
        // a dead cell comes alive if it has 1, 2, or 3 live neighbors
        // all other cells are dead
        if (count<4 && count>0 && state==0) newstate = 1;
        if (count>1 && count<4 && state==1) newstate = 1;
        
        currentworld[i][j].setState(newstate);
      }
    }
  }
  
  public void print() {
    System.out.print("+");
    for (int i = 0; i <= x*2; i++) {
      System.out.print("-");
    }
    System.out.println("+");
    for (int j = 0; j < y; j++) {
      System.out.print("|");
      if (j % 2 == 0) System.out.print(" ");
      for (int i = 0; i < x; i++) {
        int state = currentworld[i][j].getState();
        String symbol = ". ";
        if (state == 1) symbol = "X ";
        System.out.print(symbol);
      }
      if (j % 2 != 0) System.out.print(" ");
      System.out.println("|");
    }
    System.out.print("+");
    for (int i = 0; i <= x*2; i++) {
      System.out.print("-");
    }
    System.out.println("+");
  }
  
} // end class World
