// CA simulator - one-dimensional
// Jacob Peck
// 20101101

import java.util.Random;

public class WorldLinear {
  protected CellLinear[] currentworld; // current state
  protected CellLinear[] oldworld; // previous state
  protected int dim;
  
  // constructor
  public WorldLinear(int size) {
    // initialize member variables
    dim = size;
    currentworld = new CellLinear[size];
    oldworld = new CellLinear[size];
    
    // populate states
    for (int i = 0; i < dim; i++) {
      currentworld[i] = new CellLinear(0,0);
      oldworld[i] = new CellLinear(0,0);
    }
    // assign neighbors
    assignNeighbors();
  }
  
  // randomize the world
  public void randomizeWorld(int numberofstates) {
    Random random = new Random();
    for (int i = 0; i < dim; i++) {
      currentworld[i].setState(random.nextInt(numberofstates));
    }
  }
  
  // helper function for a 2-state world, such as Conway's GoL
  public void randomizeWorld() {
    randomizeWorld(2);
  }
  
  // assign neighbors to the cells
  // simple 2-neighbor neighborhood, circular
  private void assignNeighbors() {
    //System.out.println("dim = " + dim);
    for (int i = 0; i < dim; i++) {
      System.out.print(".");
      if (i > 0) {
        currentworld[i].addNeighborLeft(currentworld[i-1]);
        oldworld[i].addNeighborLeft(oldworld[i-1]);
      } else { // stitch left to right
        currentworld[i].addNeighborLeft(currentworld[dim-1]);
        oldworld[i].addNeighborLeft(oldworld[dim-1]);
      }
      if (i < dim-1) {
        currentworld[i].addNeighborRight(currentworld[i+1]);
        oldworld[i].addNeighborRight(oldworld[i+1]);
      } else { // stitch right to left
        currentworld[i].addNeighborRight(currentworld[0]);
        oldworld[i].addNeighborRight(oldworld[0]);
      }
    }
  }
  
  public void iterate() {
    System.out.println("Override this method: WorldLinear.iterate()");
  }
  
  public void print() {
    System.out.print("+");
    for (int i = 0; i < dim; i++) {
      System.out.print("-");
    }
    System.out.println("+");
    System.out.print("|");
    for (int i = 0; i < dim; i++) {
      int state = currentworld[i].getState();
      String symbol = ".";
      if (state == 1) symbol = "X";
      System.out.print(symbol);
    }
    System.out.println("|");
    System.out.print("+");
    for (int i = 0; i < dim; i++) {
      System.out.print("-");
    }
    System.out.println("+");
  }
  
} // end class World
