// Rule 30 world
// Jacob Peck
// 20101101

public class WorldLinearRule30 extends WorldLinear {
  // set of durations from whole to 1/32.
  private double values[] = {1, 0.5, 0.25, 0.125, 0.0625, 0.03125};
  
  public WorldLinearRule30(int dim) {
    super(dim);
    setValues();
  }
  
  public void iterate() {
    for (int i = 0; i < dim; i++) {
      // set history
      oldworld[i].setState(currentworld[i].getState());
    }
    
    for (int i = 0; i < dim; i++) {
      // determine whether cell lives or dies
      int state = oldworld[i].getState();
      int count = oldworld[i].livingNeighbors();
      int newstate = 0;
      
      // rule 30:
      // if cell is alive:
      // alive in next iteration if either no neighbors or just right neighbor alive
      // dead otherwise
      // if cell is dead:
      // alive in next iteration if either left or right neighbor alive, but not both
      // dead otherwise
      switch (state) {
        case 1: // alive
          if (count == 0 || count == 2) newstate = 1;
          break;
        case 0: // dead
          if (count == 1 || count == 2) newstate = 1;
          break;
        default:
          break;
      }
      currentworld[i].setState(newstate);
    }
  }
  
  private void setValues() {
    // rule 30 world is for durations
    int q = 0;
    for (int i = 0; i < dim; i++) {
      if (q >= values.length) q = 0;
      oldworld[i].setValue(values[q]);
      currentworld[i].setValue(values[q]);
      q++;
    }
  }
  
  public double getMode() { // returns the (first) mode of the set of all the values
    int counts[] = {0,0,0,0,0,0};
    for (CellLinear c: currentworld) {
      double val = c.getValue();
      int state = c.getState();
      for (int i = 0; i < values.length; i++) {
        if (val == values[i] && state == 1) {
          counts[i]++;
        }
      }
    }
    
    int index = 0;
    for (int i = 0; i < counts.length; i++) {
      if (counts[i] > counts[index]) index = i;
    }
    
    return values[index];
  }
} // end class WorldLinearRule30